package com.qmth.themis.business.templete.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.ExamStudentDtoImport;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.ExamModeEnum;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.enums.TaskTypeExecEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.templete.TaskImportTemplete;
import com.qmth.themis.business.util.ExcelCallback;
import com.qmth.themis.business.util.ExcelError;
import com.qmth.themis.business.util.ExcelUtil;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.LinkedMultiValueMap;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 考生导入模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/15
 */
public class ExamStudentTemplete implements TaskImportTemplete {
    private final static Logger log = LoggerFactory.getLogger(ExamStudentTemplete.class);

    @Override
    public Result importTask(File file, Map<String, Object> map) throws IOException, InterruptedException {
        Integer mode = Integer.parseInt(String.valueOf(map.get("mode")));
        FileInputStream inputStream = new FileInputStream(file);
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(inputStream, Lists.newArrayList(ExamStudentDtoImport.class), new ExcelCallback() {
            @Override
            public List<LinkedMultiValueMap<Integer, Object>> callback(List<LinkedMultiValueMap<Integer, Object>> finalList, List<LinkedMultiValueMap<Integer, String>> finalColumnNameList) throws IllegalAccessException {
                List<ExcelError> excelErrorList = new ArrayList<>();
                for (int i = 0; i < finalList.size(); i++) {
                    LinkedMultiValueMap<Integer, Object> map = finalList.get(i);
                    List<Object> examStudentDtoImportList = map.get(i);
                    log.info("examStudentDtoImportList.size:{}", examStudentDtoImportList.size());
                    for (int y = 0; y < examStudentDtoImportList.size(); y++) {
                        ExamStudentDtoImport examStudentDtoImport = (ExamStudentDtoImport) examStudentDtoImportList.get(y);
                        List<ExcelError> excelErrorTemp = ExcelUtil.checkExcelField(examStudentDtoImport, y, i);
                        if (excelErrorTemp.size() > 0) {
                            excelErrorList.addAll(excelErrorTemp);
                        }
                        //校验考试模式，如果是集中统一，则需填写考试场次
                        if (mode.intValue() == ExamModeEnum.together.ordinal() && Objects.isNull(examStudentDtoImport.getExamActivityCode())) {
                            excelErrorList.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行考场为空"));
                        }
                    }
                }
                if (excelErrorList.size() > 0) {
                    throw new BusinessException(JSONObject.toJSONString(excelErrorList));
                }
                return finalList;
            }
        });
        //保存数据到数据库
        if (Objects.nonNull(finalList) && finalList.size() > 0) {
            log.info("开始导入考生数据");
            long start = System.currentTimeMillis();
            TEExamStudentService teExamStudentService = SpringContextHolder.getBean(TEExamStudentService.class);
            TEStudentService teStudentService = SpringContextHolder.getBean(TEStudentService.class);
            TBUserRoleService tbUserRoleService = SpringContextHolder.getBean(TBUserRoleService.class);
            TBTaskHistoryService tbTaskHistoryService = SpringContextHolder.getBean(TBTaskHistoryService.class);
            TBExamInvigilateUserService tbExamInvigilateUserService = SpringContextHolder.getBean(TBExamInvigilateUserService.class);
            Long orgId = Long.parseLong(String.valueOf(map.get("orgId")));
            Long createId = Long.parseLong(String.valueOf(map.get("createId")));
            Long examId = Long.parseLong(String.valueOf(map.get("examId")));
            List<TEExamStudent> teExamStudentList = new ArrayList<>();
            List<TEStudent> teStudentList = new ArrayList<>();
            List<TBUserRole> tbUserRoleList = new ArrayList<>();
            List<TBExamInvigilateUser> tbExamInvigilateUserList = new ArrayList<>();
            JSONObject jsonObject = new JSONObject();
            TBTaskHistory tbTaskHistory = null;
            jsonObject.put(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"), "开始准备处理导入的考生数据");
            int y = 0, min = 0;
            try {
                Gson gson = new Gson();
                Set<String> roomCodeAndNameSet = new HashSet<>();
                Map<String, TEExamActivity> teExamActivityMap = (Map<String, TEExamActivity>) map.get("teExamActivityMap");
                Map tbTaskHistoryMap = (Map) map.get("tbTaskHistory");
                tbTaskHistoryMap = SystemConstant.timeTransform(tbTaskHistoryMap);
                tbTaskHistory = gson.fromJson(gson.toJson(tbTaskHistoryMap), TBTaskHistory.class);
                //保存用户
                for (int i = 0; i < finalList.size(); i++) {
                    LinkedMultiValueMap<Integer, Object> finalMap = finalList.get(i);
                    List<Object> examStudentDtoImportList = finalMap.get(i);
                    min = 0;
                    int max = 500, size = examStudentDtoImportList.size();
                    if (max >= size) {
                        max = size;
                    }
                    while (max <= size) {
                        List subList = examStudentDtoImportList.subList(min, max);
                        for (y = 0; y < subList.size(); y++) {
                            ExamStudentDtoImport examStudentDtoImport = (ExamStudentDtoImport) subList.get(y);
                            Map m = (Map) teExamActivityMap.get(examStudentDtoImport.getExamActivityCode());
                            m = SystemConstant.timeTransform(m);
                            TEExamActivity teExamActivity = gson.fromJson(gson.toJson(m), TEExamActivity.class);
                            //先根据证件号+科目代码查询考生是否存在，存在则更新，不存在则插入
                            QueryWrapper<TEExamStudent> teExamStudentQueryWrapper = new QueryWrapper<>();
                            teExamStudentQueryWrapper.lambda().eq(TEExamStudent::getIdentity, examStudentDtoImport.getIdentity()).eq(TEExamStudent::getCourseCode, examStudentDtoImport.getCourseCode());
                            TEExamStudent teExamStudent = teExamStudentService.getOne(teExamStudentQueryWrapper);
                            //如果为空则插入考生数据，插入考生前先插入学生档案数据
                            if (Objects.isNull(teExamStudent)) {
                                //先插入学生档案数据
                                TEStudent teStudent = new TEStudent(orgId, examStudentDtoImport.getIdentity(), examStudentDtoImport.getName(), createId);
                                teStudentService.save(teStudent);
                                teStudentList.add(teStudent);

                                //插入用户角色关系
                                TBUserRole tbUserRole = new TBUserRole(teStudent.getId(), RoleEnum.STUDENT.name());
                                tbUserRoleService.save(tbUserRole);
                                tbUserRoleList.add(tbUserRole);

                                teExamStudent = gson.fromJson(gson.toJson(examStudentDtoImport), TEExamStudent.class);
                                teExamStudent.setExamId(examId);
                                teExamStudent.setExamActivityId(teExamActivity.getId());
                                teExamStudent.setStudentId(teStudent.getId());
                                teExamStudent.setCreateId(createId);
                            } else {
                                teExamStudent.setUpdateId(createId);
                                teExamStudent.setName(examStudentDtoImport.getName());
                                teExamStudent.setCourseName(examStudentDtoImport.getCourseName());
                                teExamStudent.setGrade(examStudentDtoImport.getGrade());
                                teExamStudent.setClassNo(examStudentDtoImport.getClassNo());
                                teExamStudent.setRoomCode(examStudentDtoImport.getRoomCode());
                                teExamStudent.setRoomName(examStudentDtoImport.getRoomName());
                                teExamStudent.setExamActivityId(teExamActivity.getId());
                            }
                            roomCodeAndNameSet.add(teExamStudent.getRoomCode() + ":" + teExamStudent.getRoomName());
                            teExamStudentList.add(teExamStudent);
                        }
                        BigDecimal bigDecimal = new BigDecimal(100);
                        BigDecimal progress = new BigDecimal(Double.valueOf(new BigDecimal(max).divide(new BigDecimal(size), 2, BigDecimal.ROUND_HALF_UP).multiply(bigDecimal).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue())).setScale(0, BigDecimal.ROUND_HALF_UP);
                        teExamStudentService.saveOrUpdateBatch(teExamStudentList);
                        log.info("progress:{}", progress);
                        if (progress.intValue() == 100) {
                            tbTaskHistory.setStatus(TaskTypeExecEnum.finish.ordinal());
                            tbTaskHistory.setSummary("共处理了" + size + "条数据");
                            tbTaskHistory.setFinishTime(new Date());
                            jsonObject.put(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"), "考生数据处理完毕，" + tbTaskHistory.getSummary());
                        } else {
                            tbTaskHistory.setStatus(TaskTypeExecEnum.running.ordinal());
                            tbTaskHistory.setSummary("正在处理第" + min + "条至" + max + "条数据");
                            jsonObject.put(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"), "考生数据处理中，" + tbTaskHistory.getSummary());
                        }
                        tbTaskHistory.setProgress(progress.doubleValue());
                        tbTaskHistoryService.updateById(tbTaskHistory);
                        if (max == size) {
                            break;
                        }
                        min = max;
                        max += 500;
                        if (max >= size) {
                            max = size;
                        }
                    }
                }
                //考场创建
                if (Objects.nonNull(roomCodeAndNameSet) && roomCodeAndNameSet.size() > 0) {
                    Map<String, String> tbExamInvigilateUserMap = (Map<String, String>) map.get("tbExamInvigilateUserMap");
                    roomCodeAndNameSet.forEach(s -> {
                        if (Objects.isNull(tbExamInvigilateUserMap) || (Objects.nonNull(tbExamInvigilateUserMap) && Objects.isNull(tbExamInvigilateUserMap.get(s)))) {
                            String[] strs = s.split(":");
                            TBExamInvigilateUser tbExamInvigilateUser = new TBExamInvigilateUser(orgId, createId, strs[0], strs[1]);
                            tbExamInvigilateUserList.add(tbExamInvigilateUser);
                        }
                    });
                }
                if (Objects.nonNull(tbExamInvigilateUserList) && tbExamInvigilateUserList.size() > 0) {
                    tbExamInvigilateUserService.saveBatch(tbExamInvigilateUserList);
                    jsonObject.put(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"), "创建了" + tbExamInvigilateUserList.size() + "条考场数据");
                }
                long end = System.currentTimeMillis();
                log.info("导入考生数据结束,============耗时============:{}秒", (end - start) / 1000);
            } catch (Exception e) {
                e.printStackTrace();
                String exceptionStr = "考生数据处理到第" + (min + y + 1) + "条时发生异常:" + e.getMessage();
                jsonObject.put(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"), exceptionStr);
                tbTaskHistory.setSummary(exceptionStr);
                tbTaskHistoryService.updateById(tbTaskHistory);
                if (Objects.nonNull(teStudentList) && teStudentList.size() > 0) {
                    teStudentService.removeByIds(teStudentList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (Objects.nonNull(tbUserRoleList) && tbUserRoleList.size() > 0) {
                    tbUserRoleService.removeByIds(tbUserRoleList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (Objects.nonNull(teExamStudentList) && teExamStudentList.size() > 0) {
                    teExamStudentService.removeByIds(teExamStudentList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (Objects.nonNull(tbExamInvigilateUserList) && tbExamInvigilateUserList.size() > 0) {
                    tbExamInvigilateUserService.removeByIds(tbExamInvigilateUserList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (e instanceof BusinessException) {
                    throw new BusinessException(e.getMessage());
                } else {
                    throw new RuntimeException(e);
                }
            } finally {
                log.info("jsonObject:{}", jsonObject.toJSONString());
            }
        }
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }
}
